@@ -1,4 +1,4 @@
-{-# LANGUAGE BangPatterns, OverloadedStrings #-}
+{-# LANGUAGE BangPatterns, LambdaCase, OverloadedStrings #-}
 
 module Network.DNS.Decode.Parsers (
     getResponse
@@ -12,6 +12,7 @@ module Network.DNS.Decode.Parsers (
 
 import qualified Data.ByteString as B
 import qualified Data.ByteString.Char8 as BS
+import qualified Data.CaseInsensitive as CI
 import qualified Data.IP
 import Data.IP (IP(..), toIPv4, toIPv6b, makeAddrRange)
 
@@ -172,6 +173,10 @@ getRData SRV _ = RD_SRV <$> decodePriority
     decodePriority = get16
     decodeWeight   = get16
     decodePort     = get16
+--
+getRData RP _   = RD_RP <$> getMailbox
+                        <*> getDomain
+--
 getRData OPT len   = RD_OPT <$> getOpts len
 --
 getRData TLSA len = RD_TLSA <$> decodeUsage
@@ -277,6 +282,13 @@ getRData NSEC3PARAM _ = RD_NSEC3PARAM <$> decodeHashAlg
     decodeIterations = get16
     decodeSalt       = getInt8 >>= getNByteString
 --
+getRData CAA len = do
+    dend <- rdataEnd len
+    flags <- get8
+    tag <- getInt8 >>= getNByteString
+    tpos <- getPosition
+    RD_CAA flags (CI.mk tag) <$> getNByteString (dend - tpos)
+--
 getRData _  len = UnknownRData <$> getNByteString len
 
 ----------------------------------------------------------------
@@ -451,6 +463,23 @@ getDomain' sep1 ptrLimit = do
     let n = getValue c
     getdomain pos c n
   where
+    -- Reprocess the same ByteString starting at the pointer
+    -- target (offset).
+    getPtr pos offset = do
+        msg <- getInput
+        let parser = skipNBytes offset >> getDomain' sep1 offset
+        case runSGet parser msg of
+            Left (DecodeError err) -> failSGet err
+            Left err               -> fail $ show err
+            Right o                -> do
+                -- Cache only the presentation form decoding of domain names,
+                -- mailboxes (e.g. SOA rname) are less frequently reused, and
+                -- have a different presentation form, so must not share the
+                -- same cache.
+                when (sep1 == dot) $
+                    push pos (fst o)
+                return (fst o)
+
     getdomain pos c n
       | c == 0 = return "." -- Perhaps the root domain?
       | isPointer c = do
@@ -458,18 +487,11 @@ getDomain' sep1 ptrLimit = do
           let offset = n * 256 + d
           when (offset >= ptrLimit) $
               failSGet "invalid name compression pointer"
-          mo <- pop offset
-          case mo of
-              Nothing -> do
-                  msg <- getInput
-                  -- Reprocess the same ByteString starting at the pointer
-                  -- target (offset).
-                  let parser = skipNBytes offset >> getDomain' sep1 offset
-                  case runSGet parser msg of
-                      Left (DecodeError err) -> failSGet err
-                      Left err               -> fail $ show err
-                      Right o  -> push pos (fst o) >> return (fst o)
-              Just o -> push pos o >> return o
+          if sep1 /= dot
+              then getPtr pos offset
+              else pop offset >>= \case
+                  Nothing -> getPtr pos offset
+                  Just o  -> return o
       -- As for now, extended labels have no use.
       -- This may change some time in the future.
       | isExtLabel c = return ""
