@@ -3,10 +3,10 @@
 module EncodeSpec where
 
 import Data.IP
-import Network.DNS
-import Network.DNS.Types (defaultQuery, Question(..))
 import Test.Hspec
 
+import Network.DNS
+
 spec :: Spec
 spec = do
     describe "encode" $ do
@@ -68,8 +68,10 @@ testResponseA = DNSMessage {
          , recAvailable = True
          , rcode = NoErr
          , authenData = False
+         , chkDisable = False
          }
        }
+  , ednsHeader = NoEDNS
   , question = [Question {
                      qname = "492056364.qzone.qq.com."
                    , qtype = A
@@ -110,8 +112,10 @@ testResponseTXT = DNSMessage {
          , recAvailable = True
          , rcode = NoErr
          , authenData = False
+         , chkDisable = False
          }
        }
+  , ednsHeader = EDNSheader defaultEDNS
   , question = [Question {
                      qname = "492056364.qzone.qq.com."
                    , qtype = TXT
